using System.Collections.Generic;
using System.Linq;
using Styx;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using UI;

namespace PotionUser
{
    public static class Item
    {
        private static readonly int HealPotPercent = BeSettings.Instance.HealPotValue; // Drink HP %
        private static readonly int ManaPotPercent = BeSettings.Instance.ManaPotValue; // Drink Mana %
        private static readonly int Adds = BeSettings.Instance.AddcountValue; // Adds

        private static bool Burst
        {
            get { return StyxWoW.Me.HasAnyAura("Bloodlust", "Time Warp", "Heroism", "Ancient Hysteria"); }
        }

        private static WoWItem FindFirstUsableItemBySpell(params string[] spellNames)
        {
            List<WoWItem> carried = StyxWoW.Me.CarriedItems;
            // Yes, this is a bit of a hack. But the cost of creating an object each call, is negated by the speed of the Contains from a hash set.
            // So take your optimization bitching elsewhere.
            var spellNameHashes = new HashSet<string>(spellNames);

            return (from i in carried
                    let spells = i.ItemSpells
                    where i.ItemInfo != null && spells != null && spells.Count != 0 &&
                          i.Usable &&
                          i.Cooldown <= 0 &&
                          i.ItemInfo.RequiredLevel <= StyxWoW.Me.Level &&
                          spells.Any(
                              s => s.IsValid && s.ActualSpell != null && spellNameHashes.Contains(s.ActualSpell.Name))
                    orderby i.ItemInfo.Level descending
                    select i).FirstOrDefault();
        }

        private static WoWItem HealingPotions()
        {
            return FindFirstUsableItemBySpell("Healing Potion", "Healthstone");
        }

        private static WoWItem ManaPotions()
        {
            return FindFirstUsableItemBySpell("Restore Mana");
        }

        private static void UseHealManaPot()
        {
            if (!StyxWoW.Me.Combat || StyxWoW.Me.IsDead || StyxWoW.Me.IsGhost || StyxWoW.Me.IsOnTransport ||
                StyxWoW.Me.OnTaxi || StyxWoW.Me.Stunned || (StyxWoW.Me.Mounted && StyxWoW.Me.IsFlying)) // Chillax
            {
                return;
            }
            if (StyxWoW.Me.HealthPercent < HealPotPercent) // HP
            {
                WoWItem useHealPot = HealingPotions();
                if (useHealPot != null)
                {
                    useHealPot.UseContainerItem();
                    PuLogger.InfoLog(useHealPot.Name);
                }
            }
            if (StyxWoW.Me.ManaPercent >= ManaPotPercent) return;
            WoWItem useManaPot = ManaPotions();
            if (useManaPot == null) return;
            useManaPot.UseContainerItem();
            PuLogger.InfoLog(useManaPot.Name);
        }

        private static bool HasAnyAura(this WoWUnit unit, params string[] auraNames)
        {
            WoWAuraCollection auras = unit.GetAllAuras();
            var hashes = new HashSet<string>(auraNames);
            return auras.Any(a => hashes.Contains(a.Name));
        }

        private static void UseEquippedItem(uint slot)
        {
            WoWItem slotid = StyxWoW.Me.Inventory.Equipped.GetItemBySlot(slot);
            if (!CanUseEquippedItem(slotid)) return;
            UseItem(slotid);
        }

        private static bool CanUseEquippedItem(WoWItem item)
        {
            var itemSpell = Lua.GetReturnVal<string>("return GetItemSpell(" + item.Entry + ")", 0);
            if (string.IsNullOrEmpty(itemSpell))
                return false;

            return item.Usable && item.Cooldown <= 0;
        }

        private static void UseItemId(uint id)
        {
            foreach (WoWItem item in StyxWoW.Me.BagItems.Where(item => item.Entry == id).Where(CanUseItem))
            {
                UseItem(item);
                return;
            }
        }

        private static bool CanUseItem(WoWItem item)
        {
            return item.Usable && item.Cooldown <= 0;
        }

        private static void UseItem(WoWItem item)
        {
            PuLogger.ItemLog("Using item: " + item.Name);
            if (CanUseItem(item))
            {
                item.Use();
            }
        }

        public static void Dthdjhdrsrdzjh()
        {
            UseHealManaPot();
            CreateLifeBehaviour();
            CreateUseElixierBehavior();
            CreateUseFlaskBehavior();
            CreateUsePotionBehavior();
            CreateUseGroundedPlasmaShieldBehavior();
            CreateUseSynapseSpringsBehavior();
            CreateUseTrinket1Behavior();
            CreateUseTrinket2Behavior();
        }

        private static void CreateUseTrinket2Behavior()
        {
            if (StyxWoW.Me.CurrentTarget == null) return;
            switch (BeSettings.Instance.Trinket2Usage)
            {
                case PuEnumeration.TrinketUsage.Never:
                    return;
                case PuEnumeration.TrinketUsage.OnBoss:
                    if (!StyxWoW.Me.CurrentTarget.IsBoss()) return;
                    UseEquippedItem(13);
                    break;
                case PuEnumeration.TrinketUsage.OnCooldownInCombat:
                    UseEquippedItem(13);
                    break;
                case PuEnumeration.TrinketUsage.Questing:
                    if (StyxWoW.Me.CurrentTarget != null &&
                        StyxWoW.Me.Combat && (StyxWoW.Me.CurrentTarget.Level > StyxWoW.Me.Level ||
                                              StyxWoW.Me.HealthPercent < BeSettings.Instance.LowHealthvalue ||
                                              StyxWoW.Me.CurrentTarget.Elite ||
                                              Unit.UnfriendlyUnitsRadius.Count() >= Adds))
                        UseEquippedItem(13);
                    break;
                case PuEnumeration.TrinketUsage.LowHealth:
                    if (StyxWoW.Me.HealthPercent > BeSettings.Instance.LowHealthvalue) return;
                    UseEquippedItem(13);
                    break;
            }
        }

        private static void CreateUseTrinket1Behavior()
        {
            if (StyxWoW.Me.CurrentTarget == null) return;
            switch (BeSettings.Instance.Trinket1Usage)
            {
                case PuEnumeration.TrinketUsage.Never:
                    return;
                case PuEnumeration.TrinketUsage.OnBoss:
                    if (!StyxWoW.Me.CurrentTarget.IsBoss()) return;
                    UseEquippedItem(12);
                    break;
                case PuEnumeration.TrinketUsage.OnCooldownInCombat:
                    UseEquippedItem(12);
                    break;
                case PuEnumeration.TrinketUsage.Questing:
                    if (StyxWoW.Me.CurrentTarget != null &&
                        StyxWoW.Me.Combat && (StyxWoW.Me.CurrentTarget.Level > StyxWoW.Me.Level ||
                                              StyxWoW.Me.HealthPercent < BeSettings.Instance.LowHealthvalue ||
                                              StyxWoW.Me.CurrentTarget.Elite ||
                                              Unit.UnfriendlyUnitsRadius.Count() >= Adds))
                        UseEquippedItem(12);
                    break;
                case PuEnumeration.TrinketUsage.LowHealth:
                    if (StyxWoW.Me.HealthPercent > BeSettings.Instance.LowHealthvalue) return;
                    UseEquippedItem(12);
                    break;
            }
        }

        private static void CreateUsePotionBehavior()
        {
            if (StyxWoW.Me.CurrentTarget == null) return;
            if (!StyxWoW.Me.CurrentTarget.IsWithinMeleeRange) return;
            if (!Burst) return;
            switch (BeSettings.Instance.PotionUsage)
            {
                case PuEnumeration.PotionUsage.None:
                    return;
                case PuEnumeration.PotionUsage.GolemBloodPotion:
                    UseItemId(58146);
                    break;
                case PuEnumeration.PotionUsage.VolcanicPotion:
                    UseItemId(58091);
                    break;
                case PuEnumeration.PotionUsage.PotionofTolVir:
                    UseItemId(58145);
                    break;
                case PuEnumeration.PotionUsage.EarthenPotion:
                    UseItemId(58090);
                    break;
                case PuEnumeration.PotionUsage.VirmensBite:
                    UseItemId(76089);
                    break;
                case PuEnumeration.PotionUsage.PotionoftheMountains:
                    UseItemId(76090);
                    break;
                case PuEnumeration.PotionUsage.PotionoftheJadeSerpant:
                    UseItemId(76093);
                    break;
                case PuEnumeration.PotionUsage.PotionofMoguPower:
                    UseItemId(76095);
                    break;
            }
        }

        private static void CreateUseElixierBehavior()
        {
            switch (BeSettings.Instance.ElixierUsage)
            {
                case PuEnumeration.ElixierUsage.None:
                    return;
                case PuEnumeration.ElixierUsage.ElixirofAncientKnowledge:
                    if (StyxWoW.Me.HasAura(127250)) return;
                    UseItemId(86574);
                    break;
                case PuEnumeration.ElixierUsage.PotionofLuck:
                    if (StyxWoW.Me.HasAura(105699)) return;
                    UseItemId(76091);
                    break;
                case PuEnumeration.ElixierUsage.FlaskofEnhancement:
                    if (StyxWoW.Me.HasAura(79637)) return;
                    UseItemId(58149);
                    break;
                case PuEnumeration.ElixierUsage.FlaskoftheNorth:
                    if (StyxWoW.Me.HasAura(67019)) return;
                    UseItemId(47499);
                    break;
                case PuEnumeration.ElixierUsage.PotionofTreasureFinding:
                    if (StyxWoW.Me.HasAura(80263)) return;
                    UseItemId(58488);
                    break;
                case PuEnumeration.ElixierUsage.CrystalofInsanity:
                    if (StyxWoW.Me.HasAura(127230)) return;
                    UseItemId(86569);
                    break;
                case PuEnumeration.ElixierUsage.AlchemistsFlask:
                    if (StyxWoW.Me.HasAura(105617)) return;
                    UseItemId(75525);
                    break;
            }
        }

        private static void CreateLifeBehaviour()
        {
            if(!BeSettings.Instance.Spirit) return;
            if (!StyxWoW.Me.HasAura(130649))
                if (StyxWoW.Me.HealthPercent <= BeSettings.Instance.HealPotValue)
                    UseItemId(89640);
            if (!StyxWoW.Me.HasAura(130650))
                if (StyxWoW.Me.ManaPercent <= BeSettings.Instance.ManaPotValue)
                    UseItemId(89641);
        }

        private static void CreateUseFlaskBehavior()
        {
            switch (BeSettings.Instance.FlaskUsage)
            {
                case PuEnumeration.FlaskUsage.None:
                    return;
                case PuEnumeration.FlaskUsage.FlaskofDraconicMind:
                    if (StyxWoW.Me.HasAura(79470)) return;
                    UseItemId(58086);
                    break;
                case PuEnumeration.FlaskUsage.FlaskofFlowingWater:
                    if (StyxWoW.Me.HasAura(94160)) return;
                    UseItemId(67438);
                    break;
                case PuEnumeration.FlaskUsage.FlaskofSteelskin:
                    if (StyxWoW.Me.HasAura(79469)) return;
                    UseItemId(58085);
                    break;
                case PuEnumeration.FlaskUsage.FlaskofTitanicStrength:
                    if (StyxWoW.Me.HasAura(79472)) return;
                    UseItemId(58088);
                    break;
                case PuEnumeration.FlaskUsage.FlaskoftheWinds:
                    if (StyxWoW.Me.HasAura(79471)) return;
                    UseItemId(58087);
                    break;
                case PuEnumeration.FlaskUsage.FlaskoftheWarmSun:
                    if (StyxWoW.Me.HasAura(105691)) return;
                    UseItemId(76085);
                    break;
                case PuEnumeration.FlaskUsage.FlaskoftheEarth:
                    if (StyxWoW.Me.HasAura(105694)) return;
                    UseItemId(76087);
                    break;
                case PuEnumeration.FlaskUsage.FlaskofWintersBite:
                    if (StyxWoW.Me.HasAura(105696)) return;
                    UseItemId(76088);
                    break;
                case PuEnumeration.FlaskUsage.FlaskofSpringBlossoms:
                    if (StyxWoW.Me.HasAura(105689)) return;
                    UseItemId(76084);
                    break;
                case PuEnumeration.FlaskUsage.FlaskofFallingLeaves:
                    if (StyxWoW.Me.HasAura(105693)) return;
                    UseItemId(76086);
                    break;
                case PuEnumeration.FlaskUsage.FlaskofBattle:
                    if (StyxWoW.Me.HasAura(92679)) return;
                    UseItemId(65455);
                    break;
            }
        }

        private static void CreateUseSynapseSpringsBehavior()
        {
            if (StyxWoW.Me.CurrentTarget == null) return;
            switch (BeSettings.Instance.SsUsage)
            {
                case PuEnumeration.SynapseSpringsUsage.Never:
                    return;
                case PuEnumeration.SynapseSpringsUsage.OnBoss:
                    if (!StyxWoW.Me.CurrentTarget.IsBoss()) return;
                    UseEquippedItem(9);
                    break;
                case PuEnumeration.SynapseSpringsUsage.OnCooldownInCombat:
                    UseEquippedItem(9);
                    break;
                case PuEnumeration.SynapseSpringsUsage.WithPillarofFrost:
                    if (!StyxWoW.Me.HasAura(51271)) return;
                    UseEquippedItem(9);
                    break;
            }
        }

        private static void CreateUseGroundedPlasmaShieldBehavior()
        {
            switch (BeSettings.Instance.PlasmaUsage)
            {
                case PuEnumeration.PlasmaShieldUsage.Never:
                    return;
                case PuEnumeration.PlasmaShieldUsage.Emergency:
                    if (StyxWoW.Me.HealthPercent > 30) return;
                    UseEquippedItem(5);
                    break;
                case PuEnumeration.PlasmaShieldUsage.Under80:
                    if (StyxWoW.Me.HealthPercent > 80) return;
                    UseEquippedItem(5);
                    break;
                case PuEnumeration.PlasmaShieldUsage.Under70:
                    if (StyxWoW.Me.HealthPercent > 70) return;
                    UseEquippedItem(5);
                    break;
                case PuEnumeration.PlasmaShieldUsage.Under60:
                    if (StyxWoW.Me.HealthPercent > 60) return;
                    UseEquippedItem(5);
                    break;
                case PuEnumeration.PlasmaShieldUsage.Under50:
                    if (StyxWoW.Me.HealthPercent > 50) return;
                    UseEquippedItem(5);
                    break;
            }
        }
    }
}